import asyncio
import json
import logging
import threading

from aiogram import Bot, Dispatcher
from aiogram.types import Message
from aiogram.utils.markdown import escape_md
from apubsub import Service
from ocomone import Resources

from .parsing import CommandParsingError, parse_command
from ...configuration import OUTPUTS_CFG
from ...configuration.dynamic import DEFAULT_INSTANCE_CONFIG, InstanceConfig
from ...outputs.telegram.json2mdwn import convert
from ...sources import AWXApiSource

LOGGER = logging.getLogger(__name__)
LOGGER.setLevel(logging.DEBUG)

STATUS = 'status'
HELP = 'help'
TG_CONFIG = OUTPUTS_CFG['telegram_bot']
STATUS_TOPICS = {
    'awx': AWXApiSource.params().topic_in,
}

__MESSAGES_PATH = Resources(__file__, resources_dir='messages')


def _load_help():
    msg_path = __MESSAGES_PATH['help.md']
    with open(msg_path, encoding='utf-8') as msg_file:
        msg = ''
        for line in msg_file:
            msg += line
    return {HELP: msg}


MESSAGES = _load_help()


def _not_in_current_loop(smth):
    return (smth is None) or (asyncio.get_event_loop() != smth.loop)


async def handle_help_cmd(message: Message):
    """Handler for /help command"""
    log_msg(message)
    answer_message = MESSAGES[HELP]
    await message.answer(answer_message)


def log_msg(message: Message):
    """Add LOGGER message"""
    LOGGER.debug('Message received: %s in chat %s', message.text, message.chat)


class BotRunner:
    """Bot runner wrapper"""
    _bot: Bot = None
    _dispatcher: Dispatcher = None

    def __init__(self, msg_service: Service, stop_event: threading.Event,
                 config: InstanceConfig = DEFAULT_INSTANCE_CONFIG):
        self.client = msg_service.get_client()
        self.stop_event = stop_event
        self.config = config
        self.chat_id = config.chat_id

    @property
    def bot(self):
        """Return bot instance, create new if missing"""
        if _not_in_current_loop(self._bot):
            LOGGER.warning('No bot exist. Create bot.')
            self._bot = Bot(self.config.token,
                            parse_mode=TG_CONFIG.params['parse_mode'],
                            proxy=self.config.proxy)
        return self._bot

    @property
    def dispatcher(self):
        """Return dispatcher instance, create new if missing"""
        if _not_in_current_loop(self._dispatcher):
            LOGGER.warning('No dispatcher exist. Create dispatcher.')
            self._dispatcher = Dispatcher(self.bot)
            self._dispatcher.register_message_handler(handle_help_cmd, commands=[HELP])
            self._dispatcher.register_message_handler(self.handle_status, commands=[STATUS])
        return self._dispatcher

    async def handle_status(self, message: Message):
        """Handler for /status command"""
        log_msg(message)
        try:
            source, template_name, count = parse_command(message.text)  # pylint:disable=unused-variable
        except CommandParsingError:
            return await message.answer(escape_md('Invalid command. Please check command syntax'))
        if source in STATUS_TOPICS:
            return await self.client.publish(STATUS_TOPICS[source], template_name or '')
        return await message.answer(rf'Invalid source: `{escape_md(source)}`')

    async def alert(self, message):
        """Send message to chat alerting users"""
        LOGGER.debug('Message (alert) sent to the chat %s:\n %s', self.chat_id, message)
        return await self.bot.send_message(self.chat_id, message, disable_notification=False)

    async def silent(self, message):
        """Send message to chat without user alerting"""
        LOGGER.debug('Message (silent) sent to the chat %s:\n %s', self.chat_id, message)
        return await self.bot.send_message(self.chat_id, message, disable_notification=True)

    async def remove(self, message_id):
        """Remove message from chat"""
        LOGGER.debug('Message %s removed from chat %s', message_id, self.chat_id)
        return await self.bot.delete_message(self.chat_id, message_id)

    async def start_posting(self):
        """Start posting updates to channel"""
        await self.client.start_consuming()
        topics = TG_CONFIG.subscriptions
        await asyncio.wait([
            self.client.subscribe(topic) for topic in topics
        ])
        LOGGER.info('Bot subscribed to topics: %s', topics)
        await self.silent('__Bot started__')
        while not self.stop_event.is_set():
            # message consumed
            message = await self.client.get(.1)
            if message is not None:
                data = json.loads(message)
                try:
                    await self.alert(convert(data))  # check if TG response was 200
                except Exception:  # pylint:disable=broad-except
                    LOGGER.exception('Failed to send message %s', data)

    async def _stopper(self):
        while not self.stop_event.is_set():
            await asyncio.sleep(.1)
        self.client.stop_getting()
        self.dispatcher.stop_polling()
        await self.dispatcher.storage.close()
        await self.dispatcher.storage.wait_closed()
        await self.dispatcher.bot.close()
        await self.dispatcher.wait_closed()

    async def start(self):
        """Start bot"""
        await asyncio.wait([
            self.start_posting(),
            self.dispatcher.start_polling(),
            self._stopper(),
        ])



"""Run bot as python module"""
import asyncio
import os
import threading
from argparse import ArgumentParser

from apubsub import Service


# pylint:disable=import-outside-toplevel

def _config():
    from delatore import configuration as cfg
    arg_p = ArgumentParser(description='Bot for reporting CSM monitoring status to telegram channel and alerta')
    arg_p.add_argument('--config', default=None, help='Configuration file to use')
    arg_p.add_argument('--chat', default=os.getenv('chat_id'), help='Chat for notifications')
    arg_p.add_argument('--token', default=os.getenv('token'), help='Telegram bot token')
    arg_p.add_argument('--influx_password', default=os.getenv('INFLUX_PASSWORD'), help='Influx password')
    arg_p.add_argument('--awx_auth_token', default=os.getenv('AWX_AUTH_TOKEN'), help='OAuth2 Token for Ansible Tower')
    arg_p.add_argument('--alerta_api_key', default=os.getenv('alerta_api_key'), help='Token for Alerta')
    arg_p.add_argument('--alerta_service', default=os.getenv('alerta_service'), help='Alerta reporting service')

    args = arg_p.parse_args()
    config_file = args.config

    token, chat_id = args.token, args.chat
    influx_password, awx_auth_token = args.influx_password, args.awx_auth_token

    alerta_api_key, alerta_service = args.alerta_api_key, args.alerta_service

    if all([token, chat_id, influx_password, awx_auth_token]):
        return cfg.InstanceConfig(token, chat_id, influx_password, awx_auth_token, alerta_api_key, alerta_service)
    if config_file:
        if not os.path.isfile(config_file):
            raise FileNotFoundError
        return cfg.read_config(config_file)
    raise RuntimeError('Please provide chat ID and bot token or configuration file to use')


async def _ordered_start(service):
    from delatore.outputs import start_outputs
    from delatore.sources import start_sources

    config = _config()

    stop_event = threading.Event()
    loop = asyncio.get_running_loop()
    out_tsk = loop.create_task(start_outputs(service, stop_event, config))
    await asyncio.sleep(.5)
    src_tsk = loop.create_task(start_sources(service, stop_event, config))
    await asyncio.wait([
        src_tsk,
        out_tsk,
    ])


def _main():
    service = Service()
    service.start()

    try:
        asyncio.run(_ordered_start(service))
    finally:
        service.stop()


if __name__ == '__main__':
    _main()



"""All delatore outputs"""
import asyncio
from threading import Event

from apubsub import Service

from .alerta import AlertaRunner
from .telegram import BotRunner
from ..configuration import InstanceConfig


async def start_outputs(service: Service, stop_event: Event, config: InstanceConfig):
    await asyncio.wait([
        AlertaRunner(service, stop_event, config).start(),
        BotRunner(service, stop_event, config).start(),
    ])



import copy
import logging
from datetime import datetime
from enum import Enum
from typing import List, Optional

UNIFIED_TIME_PATTERN = '%d.%m.%Y %H:%M'

LOGGER = logging.getLogger(__name__)
LOGGER.setLevel(logging.DEBUG)


class Status(Enum):
    """Possible task statuses"""
    OK = 'ok'
    NO_DATA = 'no_data'
    FAIL = 'fail'
    RUNNING = 'running'
    CANCELED = 'canceled'

    _INVALID = 'invalid status'

    @classmethod
    def _missing_(cls, _):
        return cls._INVALID


def generate_message(source: str, status_list: List[dict]) -> dict:
    """Generate status message"""
    return {
        'source': source,
        'status_list': copy.deepcopy(status_list),
    }


def generate_error(source: str, message: str) -> dict:
    """Generate error message"""
    return {
        'source': source,
        'error': message,
    }


def generate_status(name: str,
                    status: Status,
                    timestamp: Optional[str] = None,
                    details_url: Optional[str] = None) -> dict:
    """Generate single status line"""
    return {
        'name': name,
        'status': status.value,
        'timestamp': timestamp,
        'details_url': details_url
    }


def convert_timestamp(timestamp: str, timestamp_fmt: str) -> str:
    """Convert timestamp from given format to UNIFIED_TIME_PATTERN"""
    try:
        datetime_object = datetime.strptime(timestamp, timestamp_fmt)
    except ValueError:
        LOGGER.exception('Failed to convert timestamp: %s', timestamp)
        return ''
    return datetime_object.strftime(UNIFIED_TIME_PATTERN)



import logging
from abc import ABC
from asyncio import Queue
from json import JSONDecodeError
from traceback import format_exc

from aiohttp import web
from aiohttp.web_request import Request
from apubsub.client import Client
from jsonschema import ValidationError, draft7_format_checker, validate

from .awx_api import switch_awx_status
from .base import Source
from ..configuration import DEFAULT_INSTANCE_CONFIG
from ..unified_json import convert_timestamp, generate_error, generate_message, generate_status

LOGGER = logging.getLogger(__name__)
LOGGER.setLevel(logging.DEBUG)
WH_TIME_PATTERN = '%Y-%m-%dT%H:%M:%S.%f%z'


async def ok(_: Request):
    """Send `OK` response"""
    return web.Response(text='OK')


class HttpListenerSource(Source, ABC):
    """HTTP listener"""

    # pylint: disable=abstract-method

    def __init__(self, client: Client, instance_config=DEFAULT_INSTANCE_CONFIG):
        super().__init__(client,
                         ignore_duplicates=False,
                         instance_config=instance_config)
        app = web.Application()
        app.add_routes([web.get('/', ok)])
        self.app = app
        self.port = self.config.params['port']
        self.runner = web.AppRunner(self.app)
        self.site = None

    async def start(self, stop_event):
        """Start server coroutine"""
        await self.runner.setup()
        self.site = web.TCPSite(self.runner, port=self.port)
        await self.site.start()
        LOGGER.debug('Web server started on port %s', self.port)
        await super().start(stop_event)
        await self.runner.cleanup()

    async def stop(self):
        await self.site.stop()


WEB_HOOK_JSON_SCHEMA = {
    'type': 'object',
    'properties': {
        'name': {'type': 'string'},
        'status': {'type': 'string'},
        'started': {
            'type': 'string',
            'format': 'date-time',
        },
        'url': {
            'type': ['string', 'null'],
            'format': 'url'
        }
    },
    'required': ['name', 'status', 'started', 'url'],
}


class AWXWebHookSource(HttpListenerSource):
    """HTTP listener for AWX web hooks"""

    CONFIG_ID = 'awx_web_hook'

    def __init__(self, client):
        super().__init__(client)
        self.app.add_routes([
            web.post('/notifications', self.notifications)
        ])
        self.updates = Queue()

    async def notifications(self, request: web.Request):
        try:
            data = await request.json()
        except JSONDecodeError:
            return web.Response(status=400,
                                text="Failed to parse request data as JSON")
        try:
            validate(data, WEB_HOOK_JSON_SCHEMA, format_checker=draft7_format_checker)
        except ValidationError:
            return web.Response(status=400,
                                text=f"Fail to parse data: \n{data}\n"
                                     f"Expected Schema: {WEB_HOOK_JSON_SCHEMA}")
        LOGGER.debug('Data from AWX: %s', data)
        await self.updates.put(data)
        return web.Response(text='OK')

    async def get_update(self):
        update = await self.updates.get()
        try:
            status = generate_status(
                name=update['name'],
                status=switch_awx_status(update['status']),
                timestamp=convert_timestamp(update['started'], WH_TIME_PATTERN),
                details_url=update['url'],
            )
        except KeyError:
            return generate_error(self.CONFIG_ID, format_exc(limit=10))
        return generate_message(self.CONFIG_ID, [status])



import asyncio
import json
import logging
import threading

from alertaclient.api import Client
from apubsub import Service

from ...configuration import OUTPUTS_CFG, SOURCES_CFG
from ...configuration.dynamic import DEFAULT_INSTANCE_CONFIG, InstanceConfig

LOGGER = logging.getLogger(__name__)
LOGGER.setLevel(logging.DEBUG)

ALERTA_CONFIG = OUTPUTS_CFG['alerta']
STATUS = 'status'


def get_name_timeout(conf):
    result = {}
    for metric in conf.params['metrics']:
        result[metric['name']] = metric['timeout']
    return result


INFLUX_CONFIG = get_name_timeout(SOURCES_CFG['influxdb'])


def create_msg(record):
    return f'{record["name"]} ' \
           f'has status: {record["status"]}, ' \
           f'timeout: {INFLUX_CONFIG.get(record["name"], 0)}s.'


class AlertaRunner:
    """Alerta runner wrapper"""
    _alerta: Client = None
    _alerta_thread: threading.Thread

    def __init__(self, msg_service: Service, stop_event: threading.Event,
                 config: InstanceConfig = DEFAULT_INSTANCE_CONFIG):
        self.client = msg_service.get_client()
        self.stop_event = stop_event
        self.config = config
        self.alerta_api_key = config.alerta_api_key

    @property
    def alerta(self):
        """Return alerta instance, create new if missing"""
        if self._alerta is None:
            LOGGER.warning('No alerta exist. Create client.')
            self._alerta = Client(key=self.alerta_api_key, endpoint=ALERTA_CONFIG.params['endpoint'])
        return self._alerta

    def alert(self, message):
        """Send messages to alerta"""
        alerta_ids = []
        common_args = dict(
            environment=ALERTA_CONFIG.params['environment'],
            service=[self.config.alerta_service],
            resource=f'{ALERTA_CONFIG.params["resource"]}{message["source"]}',
            origin=ALERTA_CONFIG.params['origin']
        )
        for record in message['status_list']:
            if record['status'] in ['fail', 'no_data']:
                report_id = self.alerta.send_alert(
                    event=record["name"],
                    value=create_msg(record),
                    severity=ALERTA_CONFIG.params['severity'],
                    **common_args,
                )[0]
            else:
                report_id = self.alerta.send_alert(
                    event=record["name"],
                    severity='ok',
                    **common_args,
                )[0]
            
            alerta_ids.append(report_id)
            LOGGER.debug('Alerta message sent')
        return alerta_ids

    def get_current_alerts(self, origin):
        return self.alerta.get_alerts([('origin', origin), ('repeat', False)])

    def remove(self, alerta_ids):
        """Remove message from alerta"""
        LOGGER.debug('Alerts removed from table')
        for alerta_id in alerta_ids:
            self.alerta.delete_alert(alerta_id)

    async def start_posting(self):
        """Start posting updates to channel"""
        await self.client.start_consuming()
        topics = ALERTA_CONFIG.subscriptions
        for topic in topics:
            await self.client.subscribe(topic)
        LOGGER.info('Alerta subscribed to topics: %s', topics)
        while not self.stop_event.is_set():
            # message consumed
            message = await self.client.get(.1)
            if message is not None:
                data = json.loads(message)
                self.alert(data)  # check if TG response was 200

    async def _stopper(self):
        while not self.stop_event.is_set():
            await asyncio.sleep(.5)
        await self.client.stop_getting()

    async def start(self):
        """Start alerta"""

        def _main():
            asyncio.run(self.start_posting())

        self._alerta_thread = threading.Thread(target=_main,
                                               name="Alerta-Thread", )
        self._alerta_thread.start()
        await self._stopper()
        self._alerta_thread.join()

    def stop(self, stop_event: threading.Event):
        """Stop alerta"""
        self.stop_event = stop_event



import asyncio
import json
import logging
import re
from dataclasses import dataclass
from datetime import datetime
from typing import List, NamedTuple

import aiohttp
from aiohttp_socks import ProxyConnector
from apubsub.client import Client
from influxdb import InfluxDBClient
from influxdb.exceptions import InfluxDBClientError, InfluxDBServerError
from influxdb.resultset import ResultSet

from .base import Source
from ..configuration import DEFAULT_INSTANCE_CONFIG
from ..unified_json import Status, UNIFIED_TIME_PATTERN, generate_message, generate_status

LOGGER = logging.getLogger(__name__)
LOGGER.setLevel(logging.DEBUG)


class AsyncInfluxClient(InfluxDBClient):  # pragma: no cover
    """Influx client using aiohttp instead of requests"""

    # pylint: disable=too-many-arguments

    def __init__(self, *args, proxy="", **kwargs):
        self.proxy: str = proxy
        super().__init__(*args, **kwargs)

    async def query(self,
                    query,
                    params=None,
                    bind_params=None,
                    epoch=None,
                    expected_response_code=200,
                    database=None,
                    raise_errors=True,
                    chunked=False,
                    chunk_size=0,
                    method='GET'):
        if params is None:
            params = {}

        if bind_params is not None:
            params_dict = json.loads(params.get('params', '{}'))
            params_dict.update(bind_params)
            params['params'] = json.dumps(params_dict)

        params['q'] = query
        params['db'] = database or self._database

        if epoch is not None:
            params['epoch'] = epoch

        if chunked:
            params['chunked'] = 'true'
            if chunk_size > 0:
                params['chunk_size'] = chunk_size

        if query.lower().startswith('select ') and ' into ' in query.lower():
            method = 'POST'

        data = await self.request(
            url='query',
            method=method,
            params=params,
            data=None,
            expected_response_code=expected_response_code
        )

        results = [
            ResultSet(result, raise_errors=raise_errors)
            for result
            in data.get('results', [])
        ]

        if len(results) == 1:
            return results[0]

        return results

    async def request(self, url, method='GET', params=None, data=None,
                      expected_response_code=200, headers=None):
        url = f'{self._baseurl}/{url}'

        if headers is None:
            headers = self._headers

        if params is None:
            params = {}

        if isinstance(data, (dict, list)):
            data = json.dumps(data)

        kwargs = dict(
            method=method,
            url=url,
            params=params,
            data=data,
            headers=headers,
            verify_ssl=self._verify_ssl,
            timeout=self._timeout,
        )
        if self._username is not None:
            kwargs.update(auth=aiohttp.BasicAuth(self._username, self._password or ''))

        if self.proxy:
            connector = ProxyConnector.from_url(self.proxy)
        else:
            connector = None

        async with aiohttp.ClientSession(connector=connector, headers=self._session.headers) as session:
            async with session.request(**kwargs) as response:
                data = await response.json()
        # if there's not an error, there must have been a successful response
        if 500 <= response.status < 600:
            raise InfluxDBServerError(data)
        if response.status == expected_response_code:
            return data
        raise InfluxDBClientError(data, response.status)


INFLUX_POLLING_INTERVAL = 60


@dataclass(frozen=True)
class Metric:
    """Metric description"""
    name: str
    metric_id: str
    timeout: int = 300
    query: str = ''


class InfluxParams(NamedTuple):
    """Influx params storage"""
    host: str
    port: int
    username: str
    database: str
    metrics: List[Metric]


class InfluxSource(Source):
    """InfluxDB client"""

    CONFIG_ID = 'influxdb'
    _params: InfluxParams = None

    @classmethod
    def params(cls) -> InfluxParams:
        if cls._params is None:
            params = cls.config.params
            metrics = params.pop('metrics')
            metrics = [Metric(**met) for met in metrics]
            cls._params = InfluxParams(metrics=metrics, **params)
        return cls._params

    def __init__(self, client: Client, instance_config=DEFAULT_INSTANCE_CONFIG):
        super().__init__(client, instance_config=instance_config)
        self._influx_client = None

    @property
    def influx_client(self):
        """Return new instance of influx client"""
        if self._influx_client is None:
            params = self.params()
            self._influx_client = AsyncInfluxClient(
                host=params.host,
                port=params.port,
                username=params.username,
                password=self.instance_config.influx_password,
                database=params.database,
                ssl=True,
                verify_ssl=True,
                proxy=self.instance_config.proxy,
            )
        return self._influx_client

    async def get_update(self) -> dict:
        metrics = self.params().metrics
        results = await asyncio.gather(*[
            self._get_status(met) for met in metrics
        ])
        return generate_message(self.CONFIG_ID, results)

    async def _get_status(self, metric):
        query = metric.query.format(entity=metric.metric_id)
        last_record = await self.influx_client.query(query)
        try:
            last_time = last_record.raw['series'][0]['values'][0][0]
        except KeyError:
            return generate_status(metric.name, Status.NO_DATA, None)
        now = datetime.utcnow().timestamp()
        last_time_ms = _convert_time(last_time)
        if now - last_time_ms.timestamp() < metric.timeout:
            return generate_status(metric.name, Status.OK,
                                   last_time_ms.strftime(UNIFIED_TIME_PATTERN))
        return generate_status(metric.name, Status.FAIL,
                               last_time_ms.strftime(UNIFIED_TIME_PATTERN))


class InfluxTimestampParseException(Exception):
    """Error during Influx timestamp parsing"""


_RE_TIME_GROUPS = re.compile(r'(.+)\.(\d+)(Z)')
_RE_TIME_GROUPS_WITHOUT_MS = re.compile(r'(.+)(Z)')
TIME_PATTERN = '%Y-%m-%dT%H:%M:%S.%fZ'


def _convert_time(timestamp) -> datetime:
    match_ts = _RE_TIME_GROUPS.match(timestamp)
    if match_ts is None:
        match_ts = _RE_TIME_GROUPS_WITHOUT_MS.match(timestamp)
        if match_ts is None:
            raise InfluxTimestampParseException
        timestamp, timezone = match_ts.groups()
        dtime = datetime.strptime(f'{timestamp}.123456{timezone}', TIME_PATTERN)
        return dtime
    timestamp, nsec, timezone, *_ = match_ts.groups()
    msec = nsec[:6]
    dtime = datetime.strptime(f'{timestamp}.{msec}{timezone}', TIME_PATTERN)
    return dtime



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['delatore',
 'delatore.configuration',
 'delatore.outputs',
 'delatore.outputs.alerta',
 'delatore.outputs.telegram',
 'delatore.sources']

package_data = \
{'': ['*'],
 'delatore.configuration': ['resources/*'],
 'delatore.outputs.telegram': ['messages/*']}

install_requires = \
['aiodns>=2.0.0,<3.0.0',
 'aiogram>=2.5.3,<3.0.0',
 'aiohttp-socks>=0.3.4,<0.4.0',
 'aiohttp>=3.6.2,<4.0.0',
 'alerta>=7.4.4,<8.0.0',
 'apubsub>=0.2.1,<0.3.0',
 'influxdb>=5.2.3,<6.0.0',
 'jsonschema[format]>=3.2.0,<4.0.0',
 'ocomone>=0.4.3,<0.5.0',
 'pyyaml-typed>=0.1.0,<0.2.0',
 'rfc3339-validator>=0.1.2,<0.2.0']

setup_kwargs = {
    'name': 'delatore',
    'version': '0.4.3',
    'description': 'Bot for CSM jobs notifications in telegram and alerta',
    'long_description': '# Delatore\n[![Build Status](https://travis-ci.org/opentelekomcloud-infra/delatore.svg?branch=master)](https://travis-ci.org/opentelekomcloud-infra/delatore)\n[![codecov](https://codecov.io/gh/opentelekomcloud-infra/delatore/branch/master/graph/badge.svg)](https://codecov.io/gh/opentelekomcloud-infra/delatore)\n[![PyPI version](https://img.shields.io/pypi/v/delatore.svg)](https://pypi.org/project/delatore/)\n![GitHub](https://img.shields.io/github/license/opentelekomcloud-infra/delatore)\n\nMonitor and report status of customer service monitoring scenarios\n\n## Bot commands\n\nTelegram bot accepts following commands:\n\n### `/status`\nBot reply to the message with last status(-es) retrieved from given source\n\nStatus has following syntax:\n\n`/status <source> [detailed_source] [history_depth]`\n\nIf some argument contains spaces, it should be surrounded by quotes, either `\'...\'` or `"..."`\n\n#### AWX Source\n\nStatus command for AWX source has following syntax:\n\n`/status awx [template_name] [history_depth]`\n\nExamples:\n - `/status awx` — return last job status for all _scenarios_\n - `/status awx \'Buld test host\'` — return last job status for AWX template which called \'Buld test host\'\n - `/status awx \'Scenario 1.5\' 3` — return status of last 3 jobs for AWX template which called  `Scenario 1.5`\n',
    'author': 'OTC customer service monitoring team',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/opentelekomcloud-infra/delatore',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



import asyncio
import logging
from typing import Dict, List, NamedTuple, Optional

from aiohttp import ClientSession
from apubsub.client import Client

from .base import NoUpdates, Source
from ..configuration import InstanceConfig
from ..unified_json import Status, convert_timestamp, generate_error, generate_message, generate_status

LOGGER = logging.getLogger(__name__)
LOGGER.setLevel(logging.INFO)

__awx_status_override = {
    'failed': Status.FAIL,
    'successful': Status.OK,
    'canceled': Status.CANCELED
}


def switch_awx_status(argument) -> Status:
    status = __awx_status_override.get(argument, argument)
    return Status(status)


NO_TEMPLATE_PATTERN = 'No template with name \'{}\' found'  # pylint:disable=invalid-string-quote
AWX_API_TIME_PATTERN = '%Y-%m-%dT%H:%M:%S.%fZ'


class AWXParams(NamedTuple):
    """Influx params storage"""
    host: str
    topic_in: str


class AWXApiSource(Source):
    """AWX API Client"""

    CONFIG_ID = 'awx_api'
    _params: Optional[AWXParams] = None

    async def get_update(self):
        template_name = await self.client.get(self.polling_interval)
        if template_name is None:
            raise NoUpdates

        _filter = single_template_filter(template_name)
        data = await self.get_templates(_filter)
        data = self.convert(data, template_name)
        return data

    @classmethod
    def convert(cls, data: List[Dict], template_name) -> Dict:
        if not data:
            return generate_error(cls.CONFIG_ID, NO_TEMPLATE_PATTERN.format(template_name))
        status_list = []
        for record in data:
            status_list.append(
                generate_status(
                    name=record['name'],
                    status=switch_awx_status(record['status']),
                    timestamp=convert_timestamp(record['last_job_run'], AWX_API_TIME_PATTERN),
                )
            )
        return generate_message(cls.CONFIG_ID, status_list)

    @classmethod
    def params(cls) -> AWXParams:
        if cls._params is None:
            cls._params = AWXParams(**cls.config.params)
        return cls._params

    def __init__(self, client: Client, instance_config: InstanceConfig):
        # polling here - polling of input requests
        # request timeout - timeout for API request
        super().__init__(client, ignore_duplicates=False)
        self.instance_config = instance_config

    def get_session(self):
        """Get authorized session instance"""
        return ClientSession(headers={'Authorization': f'Bearer {self.instance_config.awx_auth_token}'})

    async def start(self, stop_event: asyncio.Event):
        await self.client.start_consuming()
        await self.client.subscribe(self.params().topic_in)
        await super().start(stop_event)

    async def get_templates(self, filters: dict = None) -> Optional[list]:
        """Returns list of all job templates for csm organization

        This methods support Ansible tower filtering
        https://docs.ansible.com/ansible-tower/latest/html/towerapi/filtering.html
        """
        async with self.get_session() as session:
            async with session.get(self.params().host + '/job_templates', params=filters, timeout=5) as response:
                response_data = await response.json()
            assert response.status == 200, f'Expected response 200, got {response.status}'
        try:
            return response_data['results']
        except KeyError:
            LOGGER.error('No `results` field found in /job_templates response: \nResponse: %s', response_data)
            raise


def single_template_filter(template_name: str):
    """Filter by exact template name"""
    if template_name:
        return {'name__iexact': template_name}
    return None
